@@ -82,7 +82,7 @@ c -------------------------------------tracking a ptcl
 #include  "Zcode.h"
 
 c                next is for DEBUG
-#include  "Zcondc.h"
+
 
       real*8 cosangle
       character*70 msg
@@ -145,7 +145,7 @@ c           it should be decayed.
          endif 
          if(MoveStat .eq. Truncated) then
 c              stack current ptcl
-              call cpush(MovedTrack)
+              call cpush2(MovedTrack)
          elseif(MoveStat .eq. ToInteract ) then
             if(Zfirst .eq. 0.) then
                Zfirst = MovedTrack.pos.depth
@@ -171,6 +171,10 @@ c               update observation place
             elseif(abs(ObsPlane) .eq. perpendicular) then
                call cscalerProd(MovedTrack.vec.w, DcAtObsXyz,
      *               cosangle)
+            elseif(ObsPlane .eq. spherical) then
+               call cscalerProd(MovedTrack.vec.w, MovedTrack.pos,
+     *               cosangle)
+               cosangle = - cosangle
             else
                call 
      *         cerrorMsg('should not come here in tracking', 0)
@@ -182,20 +186,20 @@ c               update observation place
                MovedTrack.where = MovedTrack.where + 1
             endif
 c                  if more obs-site, stack current ptcl.
-            if(.not. Upgoing) then  ! downgoing
+            if(.not. Upgoing) then  ! incident is downgoing
                if(MovedTrack.where .gt. EndLevel) then
                   ! no need to stack; discard ptcl.
                else
                   MovedTrack.where =max( MovedTrack.where, 1)
-                  call cpush(MovedTrack)
+                  call cpush2(MovedTrack)
                endif
-            else    ! Upgoing
+            else    !incident is  Upgoing
                if(MovedTrack.where .lt.1 ) then
                    ! no need to stack
                else
                   MovedTrack.where =
      *                         min(MovedTrack.where, EndLevel)
-                  call cpush(MovedTrack)
+                  call cpush2(MovedTrack)
                endif
             endif
          elseif(MoveStat .eq. BorderL) then
@@ -382,6 +386,9 @@ c              convert kg/m2 into length
      *        IntInfArray(i).thickness .ne. Infty) then
                call cthick2len(h, TrackBefMove.vec.coszenith, 
      *         IntInfArray(i).thickness, leng, t, jcut)
+               if(leng .le. 0.) then
+                  leng = 1.d-3
+               endif
             else
                leng = Infty
             endif
@@ -389,7 +396,7 @@ c              convert kg/m2 into length
             leng = IntInfArray(i).length
             jcut = 0
          endif
-         if(leng .lt. minlen) then
+         if(leng .le. minlen) then
             if(jcut .ne. 0) then
                MoveStat = Truncated
             else
@@ -401,13 +408,11 @@ c              convert kg/m2 into length
             minlen = leng
          endif
       enddo
-
       if(IntInfArray(ProcessNo).decay .or. jcut .ne. 0) then
          IntInfArray(ProcessNo).thickness = clen2thick(h, 
      *      TrackBefMove.vec.coszenith, 
      *      IntInfArray(ProcessNo).length )
       endif
-
       end
 c     ***********************
 c          truncate a path of the particle, if the path
